@@ -1,8 +1,11 @@
 {-# LANGUAGE BangPatterns, CPP, OverloadedStrings #-}
+#if MIN_VERSION_ghc_prim(0,3,1)
+{-# LANGUAGE MagicHash #-}
+#endif
 
 -- |
 -- Module:      Data.Aeson.Parser.Internal
--- Copyright:   (c) 2011, 2012 Bryan O'Sullivan
+-- Copyright:   (c) 2011-2015 Bryan O'Sullivan
 --              (c) 2011 MailRank, Inc.
 -- License:     Apache
 -- Maintainer:  Bryan O'Sullivan <bos@serpentine.com>
@@ -28,29 +31,39 @@ module Data.Aeson.Parser.Internal
     , eitherDecodeStrictWith
     ) where
 
-import Data.ByteString.Builder
-  (Builder, byteString, toLazyByteString, charUtf8, word8)
-
-import Control.Applicative ((*>), (<$>), (<*), liftA2, pure)
-import Data.Aeson.Types (Result(..), Value(..))
+import Control.Applicative ((*>), (<$>), (<*), pure)
+import Control.Monad.IO.Class (liftIO)
+import Data.Aeson.Types.Internal (IResult(..), JSONPath, Result(..), Value(..))
 import Data.Attoparsec.ByteString.Char8 (Parser, char, endOfInput, scientific,
                                          skipSpace, string)
 import Data.Bits ((.|.), shiftL)
-import Data.ByteString (ByteString)
+import Data.ByteString.Internal (ByteString(..))
 import Data.Char (chr)
-import Data.Monoid (mappend, mempty)
 import Data.Text (Text)
 import Data.Text.Encoding (decodeUtf8')
-import Data.Vector as Vector (Vector, fromList)
+import Data.Text.Internal.Encoding.Utf8 (ord2, ord3, ord4)
+import Data.Text.Internal.Unsafe.Char (ord)
+import Data.Vector as Vector (Vector, empty, fromList, reverse)
 import Data.Word (Word8)
+import Foreign.ForeignPtr (withForeignPtr)
+import Foreign.Ptr (Ptr, plusPtr)
+import Foreign.Ptr (minusPtr)
+import Foreign.Storable (poke)
+import System.IO.Unsafe (unsafePerformIO)
 import qualified Data.Attoparsec.ByteString as A
 import qualified Data.Attoparsec.Lazy as L
 import qualified Data.Attoparsec.Zepto as Z
 import qualified Data.ByteString as B
+import qualified Data.ByteString.Internal as B
 import qualified Data.ByteString.Lazy as L
 import qualified Data.ByteString.Unsafe as B
 import qualified Data.HashMap.Strict as H
 
+#if MIN_VERSION_ghc_prim(0,3,1)
+import GHC.Base (Int#, (==#), isTrue#, orI#, word2Int#)
+import GHC.Word (Word8(W8#))
+#endif
+
 #define BACKSLASH 92
 #define CLOSE_CURLY 125
 #define CLOSE_SQUARE 93
@@ -67,32 +80,30 @@ import qualified Data.HashMap.Strict as H
 #define C_n 110
 #define C_t 116
 
--- | Parse a top-level JSON value.  This must be either an object or
--- an array, per RFC 4627.
+-- | Parse a top-level JSON value.
 --
 -- The conversion of a parsed value to a Haskell value is deferred
 -- until the Haskell value is needed.  This may improve performance if
 -- only a subset of the results of conversions are needed, but at a
 -- cost in thunk allocation.
+--
+-- This function is an alias for 'value'. In aeson 0.8 and earlier, it
+-- parsed only object or array types, in conformance with the
+-- now-obsolete RFC 4627.
 json :: Parser Value
-json = json_ object_ array_
+json = value
 
--- | Parse a top-level JSON value.  This must be either an object or
--- an array, per RFC 4627.
+-- | Parse a top-level JSON value.
 --
 -- This is a strict version of 'json' which avoids building up thunks
 -- during parsing; it performs all conversions immediately.  Prefer
 -- this version if most of the JSON data needs to be accessed.
+--
+-- This function is an alias for 'value''. In aeson 0.8 and earlier, it
+-- parsed only object or array types, in conformance with the
+-- now-obsolete RFC 4627.
 json' :: Parser Value
-json' = json_ object_' array_'
-
-json_ :: Parser Value -> Parser Value -> Parser Value
-json_ obj ary = do
-  w <- skipSpace *> A.satisfy (\w -> w == OPEN_CURLY || w == OPEN_SQUARE)
-  if w == OPEN_CURLY
-    then obj
-    else ary
-{-# INLINE json_ #-}
+json' = value'
 
 object_ :: Parser Value
 object_ = {-# SCC "object_" #-} Object <$> objectValues jstring value
@@ -109,8 +120,19 @@ object_' = {-# SCC "object_'" #-} do
 objectValues :: Parser Text -> Parser Value -> Parser (H.HashMap Text Value)
 objectValues str val = do
   skipSpace
-  let pair = liftA2 (,) (str <* skipSpace) (char ':' *> skipSpace *> val)
-  H.fromList <$> commaSeparated pair CLOSE_CURLY
+  w <- A.peekWord8'
+  if w == CLOSE_CURLY
+    then A.anyWord8 >> return H.empty
+    else loop H.empty
+ where
+  loop m0 = do
+    k <- str <* skipSpace <* char ':'
+    v <- val <* skipSpace
+    let !m = H.insert k v m0
+    ch <- A.satisfy $ \w -> w == COMMA || w == CLOSE_CURLY
+    if ch == COMMA
+      then skipSpace >> loop m
+      else return m
 {-# INLINE objectValues #-}
 
 array_ :: Parser Value
@@ -121,25 +143,20 @@ array_' = {-# SCC "array_'" #-} do
   !vals <- arrayValues value'
   return (Array vals)
 
-commaSeparated :: Parser a -> Word8 -> Parser [a]
-commaSeparated item endByte = do
-  w <- A.peekWord8'
-  if w == endByte
-    then A.anyWord8 >> return []
-    else loop
-  where
-    loop = do
-      v <- item <* skipSpace
-      ch <- A.satisfy $ \w -> w == COMMA || w == endByte
-      if ch == COMMA
-        then skipSpace >> (v:) <$> loop
-        else return [v]
-{-# INLINE commaSeparated #-}
-
 arrayValues :: Parser Value -> Parser (Vector Value)
 arrayValues val = do
   skipSpace
-  Vector.fromList <$> commaSeparated val CLOSE_SQUARE
+  w <- A.peekWord8'
+  if w == CLOSE_SQUARE
+    then A.anyWord8 >> return Vector.empty
+    else loop []
+  where
+    loop acc = do
+      v <- val <* skipSpace
+      ch <- A.satisfy $ \w -> w == COMMA || w == CLOSE_SQUARE
+      if ch == COMMA
+        then skipSpace >> loop (v:acc)
+        else return (Vector.reverse (Vector.fromList (v:acc)))
 {-# INLINE arrayValues #-}
 
 -- | Parse any JSON value.  You should usually 'json' in preference to
@@ -154,6 +171,7 @@ arrayValues val = do
 -- to preserve interoperability and security.
 value :: Parser Value
 value = do
+  skipSpace
   w <- A.peekWord8'
   case w of
     DOUBLE_QUOTE  -> A.anyWord8 *> (String <$> jstring_)
@@ -169,6 +187,7 @@ value = do
 -- | Strict version of 'value'. See also 'json''.
 value' :: Parser Value
 value' = do
+  skipSpace
   w <- A.peekWord8'
   case w of
     DOUBLE_QUOTE  -> do
@@ -191,60 +210,91 @@ jstring = A.word8 DOUBLE_QUOTE *> jstring_
 
 -- | Parse a string without a leading quote.
 jstring_ :: Parser Text
-jstring_ = {-# SCC "jstring_" #-} do
-  s <- A.scan False $ \s c -> if s then Just False
-                                   else if c == DOUBLE_QUOTE
-                                        then Nothing
-                                        else Just (c == BACKSLASH)
-  _ <- A.word8 DOUBLE_QUOTE
-  s1 <- if BACKSLASH `B.elem` s
-        then case Z.parse unescape s of
-            Right r  -> return r
-            Left err -> fail err
-         else return s
-
-  case decodeUtf8' s1 of
-      Right r  -> return r
-      Left err -> fail $ show err
-
 {-# INLINE jstring_ #-}
+jstring_ = {-# SCC "jstring_" #-} do
+  (s, fin) <- A.runScanner startState go
+  _ <- A.anyWord8
+  s1 <- if isEscaped fin
+        then case unescape s of
+               Right r  -> return r
+               Left err -> fail err
+        else return s
+  case decodeUtf8' s1 of
+    Right r  -> return r
+    Left err -> fail $ show err
+ where
+#if MIN_VERSION_ghc_prim(0,3,1)
+    isEscaped (S _ escaped) = isTrue# escaped
+    startState              = S 0# 0#
+    go (S a b) (W8# c)
+      | isTrue# a                     = Just (S 0# b)
+      | isTrue# (word2Int# c ==# 34#) = Nothing   -- double quote
+      | otherwise = let a' = word2Int# c ==# 92#  -- backslash
+                    in Just (S a' (orI# a' b))
 
-unescape :: Z.Parser ByteString
-unescape = toByteString <$> go mempty where
-  go acc = do
+data S = S Int# Int#
+#else
+    isEscaped (S _ escaped) = escaped
+    startState              = S False False
+    go (S a b) c
+      | a                  = Just (S False b)
+      | c == DOUBLE_QUOTE  = Nothing
+      | otherwise = let a' = c == backslash
+                    in Just (S a' (a' || b))
+      where backslash = BACKSLASH
+
+data S = S !Bool !Bool
+#endif
+
+unescape :: ByteString -> Either String ByteString
+unescape s = unsafePerformIO $ do
+  let len = B.length s
+  fp <- B.mallocByteString len
+  -- We perform no bounds checking when writing to the destination
+  -- string, as unescaping always makes it shorter than the source.
+  withForeignPtr fp $ \ptr -> do
+    ret <- Z.parseT (go ptr) s
+    case ret of
+      Left err -> return (Left err)
+      Right p -> do
+        let newlen = p `minusPtr` ptr
+            slop = len - newlen
+        Right <$> if slop >= 128 && slop >= len `quot` 4
+                  then B.create newlen $ \np -> B.memcpy np ptr newlen
+                  else return (PS fp 0 newlen)
+ where
+  go ptr = do
     h <- Z.takeWhile (/=BACKSLASH)
     let rest = do
           start <- Z.take 2
           let !slash = B.unsafeHead start
               !t = B.unsafeIndex start 1
-              escape = case B.findIndex (==t) "\"\\/ntbrfu" of
+              escape = case B.elemIndex t "\"\\/ntbrfu" of
                          Just i -> i
                          _      -> 255
           if slash /= BACKSLASH || escape == 255
             then fail "invalid JSON escape sequence"
-            else do
-            let cont m = go (acc `mappend` byteString h `mappend` m)
-                {-# INLINE cont #-}
+            else
             if t /= 117 -- 'u'
-              then cont (word8 (B.unsafeIndex mapping escape))
+              then copy h ptr >>= word8 (B.unsafeIndex mapping escape) >>= go
               else do
                    a <- hexQuad
                    if a < 0xd800 || a > 0xdfff
-                     then cont (charUtf8 (chr a))
+                     then copy h ptr >>= charUtf8 (chr a) >>= go
                      else do
                        b <- Z.string "\\u" *> hexQuad
                        if a <= 0xdbff && b >= 0xdc00 && b <= 0xdfff
                          then let !c = ((a - 0xd800) `shiftL` 10) +
                                        (b - 0xdc00) + 0x10000
-                              in cont (charUtf8 (chr c))
+                              in copy h ptr >>= charUtf8 (chr c) >>= go
                          else fail "invalid UTF-16 surrogates"
     done <- Z.atEnd
     if done
-      then return (acc `mappend` byteString h)
+      then copy h ptr
       else rest
   mapping = "\"\\/\n\t\b\r\f"
 
-hexQuad :: Z.Parser Int
+hexQuad :: Z.ZeptoT IO Int
 hexQuad = do
   s <- Z.take 4
   let hex n | w >= C_0 && w <= C_9 = w - C_0
@@ -271,25 +321,25 @@ decodeStrictWith :: Parser Value -> (Value -> Result a) -> B.ByteString
 decodeStrictWith p to s =
     case either Error to (A.parseOnly p s) of
       Success a -> Just a
-      Error _ -> Nothing
+      _         -> Nothing
 {-# INLINE decodeStrictWith #-}
 
-eitherDecodeWith :: Parser Value -> (Value -> Result a) -> L.ByteString
-                 -> Either String a
+eitherDecodeWith :: Parser Value -> (Value -> IResult a) -> L.ByteString
+                 -> Either (JSONPath, String) a
 eitherDecodeWith p to s =
     case L.parse p s of
-      L.Done _ v -> case to v of
-                      Success a -> Right a
-                      Error msg -> Left msg
-      L.Fail _ _ msg -> Left msg
+      L.Done _ v     -> case to v of
+                          ISuccess a      -> Right a
+                          IError path msg -> Left (path, msg)
+      L.Fail _ _ msg -> Left ([], msg)
 {-# INLINE eitherDecodeWith #-}
 
-eitherDecodeStrictWith :: Parser Value -> (Value -> Result a) -> B.ByteString
-                       -> Either String a
+eitherDecodeStrictWith :: Parser Value -> (Value -> IResult a) -> B.ByteString
+                       -> Either (JSONPath, String) a
 eitherDecodeStrictWith p to s =
-    case either Error to (A.parseOnly p s) of
-      Success a -> Right a
-      Error msg -> Left msg
+    case either (IError []) to (A.parseOnly p s) of
+      ISuccess a      -> Right a
+      IError path msg -> Left (path, msg)
 {-# INLINE eitherDecodeStrictWith #-}
 
 -- $lazy
@@ -321,6 +371,37 @@ jsonEOF = json <* skipSpace <* endOfInput
 jsonEOF' :: Parser Value
 jsonEOF' = json' <* skipSpace <* endOfInput
 
-toByteString :: Builder -> ByteString
-toByteString = L.toStrict . toLazyByteString
-{-# INLINE toByteString #-}
+word8 :: Word8 -> Ptr Word8 -> Z.ZeptoT IO (Ptr Word8)
+word8 w ptr = do
+  liftIO $ poke ptr w
+  return $! ptr `plusPtr` 1
+
+copy :: ByteString -> Ptr Word8 -> Z.ZeptoT IO (Ptr Word8)
+copy (PS fp off len) ptr =
+  liftIO . withForeignPtr fp $ \src -> do
+    B.memcpy ptr (src `plusPtr` off) len
+    return $! ptr `plusPtr` len
+
+charUtf8 :: Char -> Ptr Word8 -> Z.ZeptoT IO (Ptr Word8)
+charUtf8 ch ptr
+  | ch < '\x80'   = liftIO $ do
+                       poke ptr (fromIntegral (ord ch))
+                       return $! ptr `plusPtr` 1
+  | ch < '\x800'  = liftIO $ do
+                       let (a,b) = ord2 ch
+                       poke ptr a
+                       poke (ptr `plusPtr` 1) b
+                       return $! ptr `plusPtr` 2
+  | ch < '\xffff' = liftIO $ do
+                       let (a,b,c) = ord3 ch
+                       poke ptr a
+                       poke (ptr `plusPtr` 1) b
+                       poke (ptr `plusPtr` 2) c
+                       return $! ptr `plusPtr` 3
+  | otherwise     = liftIO $ do
+                       let (a,b,c,d) = ord4 ch
+                       poke ptr a
+                       poke (ptr `plusPtr` 1) b
+                       poke (ptr `plusPtr` 2) c
+                       poke (ptr `plusPtr` 3) d
+                       return $! ptr `plusPtr` 4
