@@ -1,8 +1,6 @@
-{-# LANGUAGE CPP #-}
-
 -- |
 -- Module:      Data.Aeson.Types
--- Copyright:   (c) 2011, 2012 Bryan O'Sullivan
+-- Copyright:   (c) 2011-2015 Bryan O'Sullivan
 --              (c) 2011 MailRank, Inc.
 -- License:     Apache
 -- Maintainer:  Bryan O'Sullivan <bos@serpentine.com>
@@ -15,6 +13,9 @@ module Data.Aeson.Types
     (
     -- * Core JSON types
       Value(..)
+    , Encoding
+    , fromEncoding
+    , Series
     , Array
     , emptyArray
     , Pair
@@ -32,15 +33,15 @@ module Data.Aeson.Types
     , parseEither
     , parseMaybe
     , ToJSON(..)
+    , KeyValue(..)
     , modifyFailure
 
-#ifdef GENERICS
     -- ** Generic JSON classes
     , GFromJSON(..)
     , GToJSON(..)
     , genericToJSON
+    , genericToEncoding
     , genericParseJSON
-#endif
 
     -- * Inspecting @'Value's@
     , withObject
@@ -50,8 +51,8 @@ module Data.Aeson.Types
     , withScientific
     , withBool
 
-    -- * Constructors and accessors
-    , (.=)
+    , pairs
+    , foldable
     , (.:)
     , (.:?)
     , (.!=)
@@ -60,13 +61,13 @@ module Data.Aeson.Types
     -- * Generic and TH encoding configuration
     , Options(..)
     , SumEncoding(..)
+    , camelTo
+    , camelTo2
     , defaultOptions
     , defaultTaggedObject
     ) where
 
+import Data.Aeson.Encode.Functions (foldable, pairs)
+import Data.Aeson.Types.Generic ()
 import Data.Aeson.Types.Instances
 import Data.Aeson.Types.Internal
-
-#ifdef GENERICS
-import Data.Aeson.Types.Generic ()
-#endif
