@@ -7,19 +7,15 @@
 {-# LANGUAGE OverloadedStrings #-}
 
 import Data.Aeson (decode, encode)
-import Data.Aeson.TH (deriveJSON)
+import Data.Aeson.TH (deriveJSON, defaultOptions)
 import qualified Data.ByteString.Lazy.Char8 as BL
 
 data Coord = Coord { x :: Double, y :: Double }
              deriving (Show)
 
 -- This splice will derive instances of ToJSON and FromJSON for us.
---
--- The use of "id" below is a placeholder function to transform the
--- names of the type's fields.  We don't want to transform them, so we
--- use the identity function.
 
-$(deriveJSON id ''Coord)
+$(deriveJSON defaultOptions ''Coord)
 
 main :: IO ()
 main = do
