@@ -1,8 +1,5 @@
-{-# Language CPP, OverloadedStrings, RecordWildCards, StandaloneDeriving #-}
+{-# Language OverloadedStrings, RecordWildCards, StandaloneDeriving #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
-#if __GLASGOW_HASKELL__ < 702
-{-# OPTIONS_GHC -fno-warn-missing-methods #-}
-#endif
 
 module Instances where
 
@@ -11,8 +8,10 @@ import Data.Function (on)
 import Control.Monad
 import Test.QuickCheck (Arbitrary(..), Gen, choose, oneof, elements)
 import Data.Time.Clock (DiffTime, UTCTime(..), picosecondsToDiffTime)
+import Data.Fixed (Pico)
 import Data.Time (ZonedTime(..), LocalTime(..), TimeZone(..),
-                  hoursToTimeZone, Day(..), TimeOfDay(..))
+                  hoursToTimeZone, Day(..), TimeOfDay(..),
+                  NominalDiffTime)
 import qualified Data.Text as T
 import qualified Data.Map as Map
 import Data.Text (Text)
@@ -52,6 +51,9 @@ instance Arbitrary DotNetTime where
 instance Arbitrary ZonedTime where
     arbitrary = liftM2 ZonedTime arbitrary arbitrary
 
+instance Arbitrary NominalDiffTime where
+    arbitrary = realToFrac <$> (arbitrary :: Gen Pico)
+
 deriving instance Eq ZonedTime
 
 -- Compare equality to within a millisecond, allowing for rounding
@@ -145,3 +147,9 @@ instance Arbitrary a => Arbitrary (SomeType a) where
                       , Product <$> arbitrary <*> arbitrary <*> arbitrary
                       , Record  <$> arbitrary <*> arbitrary <*> arbitrary
                       ]
+
+instance ApproxEq Char where
+    (=~) = (==)
+
+instance (ApproxEq a) => ApproxEq [a] where
+    a =~ b = length a == length b && all (uncurry (=~)) (zip a b)
